"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const twing_1 = require("twing");
class Transpiler {
    constructor() {
        this._functions = new Set();
    }
    transpileCommentNode(node) {
        let results = [];
        let parts = node.getAttribute("data").split('\n');
        results.push('<?php /*');
        let index = 0;
        for (let part of parts) {
            results.push(part);
            if (index < (parts.length - 1)) {
                results.push('\n');
            }
            index++;
        }
        results.push('*/ ?>');
        return results.join('');
    }
    transpileExpressionBinaryNode(node) {
        return this.transpileNode(node.getNode('left')) + '==' + this.transpileNode(node.getNode('right'));
    }
    transpileExpressionNameNode(node) {
        return `$${node.getAttribute('name')}`;
    }
    transpileExpressionAssignNameNode(node) {
        return node.getAttribute('name');
    }
    transpileExpressionConstantNode(node, raw) {
        let value = node.getAttribute('value');
        if (!raw && typeof value === 'string') {
            value = `"${value}"`;
        }
        return String(value);
    }
    transpileExpressionGetAttrNode(node) {
        let results = [];
        results.push(this.transpileNode(node.getNode('node')));
        let type = node.getAttribute('type');
        let attribute = node.getNode('attribute');
        if (type === twing_1.TwingTemplate.ANY_CALL || type == twing_1.TwingTemplate.METHOD_CALL) {
            results.push('->');
            results.push(this.transpileNode(attribute, true));
        }
        if (type === twing_1.TwingTemplate.ARRAY_CALL) {
            results.push('[');
            results.push(this.transpileNode(attribute));
            results.push(']');
        }
        if (type === twing_1.TwingTemplate.METHOD_CALL) {
            results.push('(');
            // we can remove explicit typing when https://github.com/NightlyCommit/twing/issues/476 is fixed
            let argumentsNode = node.getNode('arguments');
            let argumentsPairs = argumentsNode.getKeyValuePairs();
            let _arguments = [];
            for (let pair of argumentsPairs) {
                _arguments.push(this.transpileNode(pair.value));
            }
            results.push(_arguments.join(','));
            results.push(')');
        }
        return results.join('');
    }
    transpileExpressionArrayNode(node) {
        let results = [];
        results.push('[');
        // we can remove explicit typing when https://github.com/NightlyCommit/twing/issues/476 is fixed
        let pairs = node.getKeyValuePairs();
        let values = [];
        for (let pair of pairs) {
            values.push(this.transpileNode(pair.key) + '=>' + this.transpileNode(pair.value));
        }
        results.push(values.join(','));
        results.push(']');
        return results.join('');
    }
    transpileForNode(node) {
        let results = [];
        results.push('<?php foreach (');
        results.push(this.transpileNode(node.getNode('seq')));
        results.push(' as ');
        results.push(`$${this.transpileNode(node.getNode('key_target'))}`);
        results.push(' => ');
        results.push(`$${this.transpileNode(node.getNode('value_target'))}`);
        results.push('): ?>');
        results.push(this.transpileNode(node.getNode('body')));
        results.push('<?php endforeach; ?>');
        return results.join('');
    }
    transpileIfNode(node) {
        let results = [];
        let testNodes = node.getNode('tests').getNodes();
        let testIndex = 0;
        for (let [index, testNode] of testNodes) {
            if (index === 0) {
                results.push('<?php if (');
            }
            else if (index % 2 === 0) {
                results.push('<?php elseif (');
            }
            if (index % 2 === 0) {
                results.push(this.transpileNode(testNode));
                results.push('): ?>');
            }
            else {
                results.push(this.transpileNode(testNode, true));
            }
            testIndex++;
        }
        if (node.hasNode('else')) {
            results.push('<php else: ?>');
            results.push(this.transpileNode(node.getNode('else'), true));
        }
        results.push('<?php endif; ?>');
        return results.join('');
    }
    transpilePrintNode(node) {
        return `<?=${this.transpileNode(node.getNode('expr'))}?>`;
    }
    transpileTextNode(node) {
        return node.getAttribute('data');
    }
    transpileExpressionFunctionNode(node) {
        let argumentsNode = node.getNode('arguments');
        let parameters = [...argumentsNode.getNodes().values()].map((value) => {
            return this.transpileNode(value);
        });
        return `${node.getAttribute('name')}(${parameters.join(',')})`;
    }
    transpileNode(node, raw = false) {
        if (node.getType() === twing_1.TwingNodeType.PRINT) {
            return this.transpilePrintNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.TEXT) {
            return this.transpileTextNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_NAME) {
            return this.transpileExpressionNameNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_CONSTANT) {
            return this.transpileExpressionConstantNode(node, raw);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_ARRAY) {
            return this.transpileExpressionArrayNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_ASSIGN_NAME) {
            return this.transpileExpressionAssignNameNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.FOR) {
            return this.transpileForNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.IF) {
            return this.transpileIfNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_GET_ATTR) {
            return this.transpileExpressionGetAttrNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_BINARY) {
            return this.transpileExpressionBinaryNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.COMMENT) {
            return this.transpileCommentNode(node);
        }
        if (node.getType() === twing_1.TwingNodeType.EXPRESSION_FUNCTION) {
            return this.transpileExpressionFunctionNode(node);
        }
        let results = [];
        for (let [name, child] of node.getNodes()) {
            results.push(this.transpileNode(child));
        }
        return results.join('');
    }
    transpile(code) {
        let env = new twing_1.TwingEnvironment(new twing_1.TwingLoaderArray({}), {
            autoescape: false
        });
        for (let functionName of this._functions) {
            env.addFunction(new twing_1.TwingFunction(functionName, null, []));
        }
        let node = env.parse(env.tokenize(new twing_1.TwingSource(code, '')));
        return this.transpileNode(node);
    }
    /**
     * Register a function to the transpiler.
     *
     * @param name
     */
    registerFunction(name) {
        this._functions.add(name);
    }
}
exports.Transpiler = Transpiler;
